-- vertex

#version 330 core

#define	PI					3.1415926

uniform mat4  mv;
uniform mat4  proj;
uniform vec3  eye;
uniform float t;
uniform int   nx, ny;
uniform int   gridSize;

uniform sampler3D noiseMap;

layout ( location = 0 ) in vec4 pos;
layout ( location = 1 ) in vec2 texCoord;

out vec4 tex;

	// remap instance id into box id (assume 3x3x3 boxes)
ivec3	getBoxIndex ()
{
	int	ix = gl_InstanceID % 3;
	int	ii = gl_InstanceID / 3;
	int	iy = ii % 3;
	int	iz = ii / 3;

	return ivec3 ( ix - 1, iy - 1, iz - 1 );
}

void main(void)
{
		// get grid box index with camera in it
	vec3  org   = vec3 ( gridSize ) * floor ( eye / vec3 ( gridSize ) );
//vec3 org = vec3 ( 0 );

		// get box origin
	ivec3	boxIndex = getBoxIndex ();
	vec3	offset   = vec3 ( gridSize ) * vec3 ( boxIndex );
	
		// grass texture coordinate for this vertex
	tex         = vec4 ( texCoord, pos.zw );
	gl_Position = proj * mv * vec4 ( gridSize * pos.xyz + org + offset, 1.0 );
}

-- geometry

#version 330 core

layout ( points ) in;
layout ( line_strip, max_vertices = 8 ) out;

uniform	float threshold = 0.1;

in  vec4 tex [];
out vec4 texOut;

void	main ()
{
	vec4	pos = gl_in [0].gl_Position;
	vec4	upDir = vec4 ( 0, 1, 0, 0 );

	if ( tex [0].w > threshold )
		return;

	gl_Position = pos;
	EmitVertex ();

	gl_Position = pos + 0.2 * upDir;
	EmitVertex ();
	EndPrimitive ();
}

-- fragment

#version 330 core

uniform sampler2D grassMap;

in vec4 texOut [];

out vec4  color;

void main(void)
{
//	color = vColor  * texture ( grassMap, tex );
	color = vec4 ( 0, 1, 0, 1 );
}
